from __future__ import division
import numpy as np
import math
from scipy.optimize import newton
from numpy import linalg as la


"""

The Function 'calendar' computes an angular calendar for  given orbital parameters. 
Each month occupies a 30 degrees of earth's true anomaly, while seasons are defined by 90 degree parts. The calendar is aligned by setting to Northern Hemisphere Vernal equinox at March 21st at noon. The year length is set at 365 days.

input parameter: 
eccentricity 'epsilon'
longitude of perihelion (in degree) 'VE' 
desired timesteps: 'seasons' or 'months'

output parameter:  array containing the season/month lengths

----------------------
The results should be:
----------------------

1. SEASONS

0 ka
	integer		exact
	length		length
1	93		92.79363545
2	93		93.38276473
3	90		89.81128047
4	89		89.01231936


126 ka
	integer		exact
	length		length
1	89		88.67982691
2	85		85.38952413
3	94		93.753021
4	97		97.17762796


6 ka
	integer		exact
	length		length
1	93		93.30014118
2	89		89.20745719
3	89		89.07857008
4	94		93.41383156



2. MONTHS

0 ka
	integer		exact 
	length		length
1	29		29.46881765
2	30		29.70581268
3	30		30.13819417
4	31		30.65805561
5	31		31.126942
6	31		31.4117961
7	31		31.17799177
8	31		31.17063134
9	31		30.71644531
10	30		30.19513792
11	30		29.7466091
12	30		29.48356636


126 ka
	integer		exact 
	length		length
1	33		32.64390139
2	32		32.45797409	
3	31		31.46157691
4	30		30.22403980
5	29		29.09306417
6	28		28.33917973
7	28		28.11780182
8	29		28.47080283
9	29		29.33060579
10	31		30.51440924
11	32		31.72805002
12	33		32.61859422


6 ka
	integer		exact 
	length		length
1	31		30.87904201	
2	31		31.34776949
3	31		31.3193606
4	32		31.47651396
5	31		31.09764016
6	31		30.54551351
7	30		29.97539917
8	30		29.53633743
9	29		29.33531143
10	29		29.41904421
11	30		29.76824664
12	30		30.29982139


"""


def calendar(epsilon,VE,timestep):
		

	VE = VE*2*math.pi/360
	VE = 2*math.pi-VE
	VE_mod = 282.157*2*math.pi/360
		
	if timestep == 'seasons':
		times = np.zeros(5)
		laengen_dec = np.zeros(4)
		angles = np.arange(0,2*math.pi+math.pi/2,math.pi/2)

	elif timestep == 'months':
		times = np.zeros(13)
		laengen_dec = np.zeros(12)
		angles = np.arange(0,2*math.pi+math.pi/6,math.pi/6)

	else:
		print('invalid input')
	
	n = times.size


	"""
	Two fundamental functions: 'day' and 'ang'.
	
	'day': For a given angle (seen from perihelion), 'day' returns the time elapsed since perihelion.
	'ang': For a given time (seen from perihelion), 'ang' returns the angle (seen from perihelion.

	"""

	def day(phi):

		exzAn = 2*np.arctan(((1-epsilon)/(1+epsilon))**0.5*np.tan(phi/2))
		M = exzAn - epsilon*math.sin(exzAn)
		t = M*365.25/(2*math.pi)

		return t

	def ang(t,t0=0):

		M = 2*math.pi/365.25*(t-t0)

		def exz(E):
			return E-epsilon*math.sin(E)-M
		def exzprime(E):
			return 1-epsilon*math.cos(E)
		def exz2prime(E):
			return epsilon*math.cos(E)

		exzAn = newton(exz,2*math.pi*(t-t0)/365.25,fprime=exzprime,tol = 10**-5,fprime2=exz2prime)
		phi = 2*np.arctan(((1+epsilon)/(1-epsilon))**0.5*math.tan(exzAn/2))

		return phi


	"""
	Shift to April 1st:

	For calculating the months, the starting day has to be shifted from the day of the VE (fixed at March 21st) to April 1st. 
	Since the time between nowadays March 21st and April 1st may not be true for past calendars, 
	we define April 1st by the angle. 
	Therefore we calculate the angle between nowadays March 21st, noon and the 10.5 days-later-April 1st and afterwards
	calculate the day of April 1st for the past calendar.

	"""
	
	tVE_mod = day(VE_mod) #day of modern VE
	phi14_mod = ang(tVE_mod+10.5)-VE_mod #angle between modern VE and April 1st


	tVE = day(VE) #time of past VE
	t14 = day(VE+phi14_mod)	#time of past April 1st, defined by the angle

		

	#shifts the days s.t. the first month starts at April 1st
	for w in range(n):

		times[w] = day(VE+angles[w]) - tVE
		
		if timestep == 'months':

			times[w] = day(VE+phi14_mod+angles[w]) - tVE - t14


	#from month starting day April 1st, calculate monthly lengths. Nothing has been rounded yet.
	for i in range(n-1):

		laengen_dec[i] = times[i+1]-times[i]
		laengen_dec[i] = laengen_dec[i]%365


	#change to right order: January, February, ...
	if timestep == 'months':
		laengen_dec = laengen_dec[[9,10,11,0,1,2,3,4,5,6,7,8]]
	

	"""
	Use the 'largest remainder method' for rounding: Each month gets his 'interger-part'-number of days, the remaining
	(365-sum of all integer part) days are distributed by the size of the month's decimal parts.
	
	Doing this, every year gets 365 days and they are reasonable distributed.
	"""

	laengen_int,dec = divmod(laengen_dec,1)
	ges = int(sum(laengen_int))

	for i in range(365-ges):

		k = np.argmax(dec)
		laengen_int[k] = laengen_int[k]+1
		ges = ges+1
		dec[k] = 0


	return laengen_int,laengen_dec



#test: Compute angular calendar for today (tod), 126 ka and 6 ka using the orbital parameters from Berger's routine.


caltods = calendar(0.016724,282.157,'seasons')
caltodm = calendar(0.016724,282.157,'months')

cal126s = calendar(0.039726,112.133,'seasons')
cal126m = calendar(0.039726,112.133,'months')

cal6s = calendar(0.018670,181.750,'seasons')
cal6m = calendar(0.018670,181.750,'months')

print('Seasons')
print(caltods)
print(cal126s)
print(cal6s)

print('Months')
print(caltodm)
print(cal126m)
print(cal6m)

