
from __future__ import division
from scipy.io import netcdf as nc
import scipy.sparse
import os
import sys
import numpy as np
import argparse
import xarray as xr
from matplotlib import pyplot as plt
from numpy import linalg as la


"""
Python Script for executing the calendar conversion method.
The script is executed using the shell script newmean_bash.

A parser is created, getting 12 monthly files per year.

The subfunctions calcal, meanpresalgo and meanpres_transform are described in detail below.

Output netcdf files:

meanpres_$year (Contains the new means, calculated using the mean preserving method)
"""




def calcal(caltod,calpast):

	"""
	A few calendar calculations: In 'bias' you find the calendar Offset.
	For example: bias[0] = -7 means that past's January started 7 days
	earlier w.r.t to the present calendar.
	In timtod/timtoday you find the absolute starting days, again w.r.t to the present calendar.
	Timpast can look like [-7,23,52,...], while timtod looks like [0,31,59,...]. 
	"""

	bias = np.zeros(12)
	timtod = np.zeros(13)
	timpast = np.zeros(13)

	bias[0] = caltod[0]+caltod[1]+caltod[2]-(calpast[0]+calpast[1]+calpast[2])

	for i in range(1,12):
		bias[i] = (bias[i-1]+calpast[i-1])-caltod[i-1]

	for i in range(1,13):
		timtod[i] = sum(caltod[:i])
		timpast[i] = sum(calpast[:i])

	timpast = timpast+bias[0]
	

	return bias,timtod,timpast





def meanpresalgo(caltod,calpast,timtod,timpast,mean,leapyear):

	"""
	MEAN PRESERVING ALGORITHM METHOD

	Daily data is reconstructed from monthly means. 
	For each day, the algorithm computed recursively the average of each day and the two adjacent days. 
	The daily data is initialised as being the monthly average of their respective month and 
	then updated after each step. 
	After 365 iterations this results in a nicely smooth annual cycle.
	In order to additionally keep the old monthly means, we add a correction term to each daily value.

	"""

	#modification for leapyears
	if leapyear == 'yes':
		var = 366
	else:
		var = 365

	#create tridiagonal matrix A for meanpres
	v = 1/3*np.ones(var)
	w = 1/3*np.ones(var-1)
	A = scipy.sparse.diags([v,w,w],[0,1,-1]).toarray()


	if mean[0] == mean[11]:
		A[0,var-1] = 1/3
		A[var-1,0] = 1/3
	else:
		A[0,0] = 1/2
		A[0,1] = 1/2
		A[var-1,var-1] = 1/2
		A[var-1,var-2] = 1/2

	#MO as old data, MN as new data
	MO = np.zeros(var)
	MN = np.zeros(var)
	C = np.zeros(12)

	i0 = 0
	i1 = caltod[0]

	#initialise MO as old monthly means
	for i in range(12):

		MO[i0:i1] += mean[i]
	
		if i<11:

			i0 += caltod[i]
			i1 += caltod[i+1]

	for i in range(var):

		#calculate new daily data
		MN = A.dot(MO)

		for K in range(12):

			tim = int(timtod[K])
			tim1 = int(timtod[K+1])

			#calculate correction term C per month K
			C[K] = mean[K]-sum(MN[tim:tim1])/caltod[K]

			#add correction term
			MN[tim:tim1] += C[K]

		MO = MN	#update MO


	#calculate new monthly means according to angular calendar, using the smoothed data
	newmean = np.zeros(12)

	for i in range(12):

		i0 = int(timpast[i]%var)
		i1 = int(timpast[i+1]%var)
		
		if i0 > i1:
			newmean[i] = (sum(MN[i0:])+sum(MN[:i1]))/calpast[i]
		else:
			newmean[i] = sum(MN[i0:i1])/calpast[i]

	return newmean



def meanpres_transform(caltod,calpast,timtod,timpast,leapyear):

	"""
	Apply the conversion method in meanpresalgo to each unit vector (vector being '1' in one month and '0' in all other months). 
	As a result, get each column of the conversion matrix.

	Input parameter: Calendar information (caltod,calast,timtod,timpast,leapyear)
	Output parameter: Climate-data independent conversion matrix
	"""

	mittel_trafo = np.zeros(12)

	#define the conversion matrix 2 by applying the meanpres procedure to the initvector, being 1 in one month and 0 in all others.
	convmatrix = np.zeros([12,12])

	for i in range(12):

		initvektor = np.zeros(12)
		initvektor[i] = 1
		convmatrix[:,i] = meanpresalgo(caltod,calpast,timtod,timpast,initvektor,leapyear)

	return convmatrix




desc = """\
Parser for reading monthly data
"""


parser = argparse.ArgumentParser(description=desc, formatter_class = argparse.RawDescriptionHelpFormatter)

parser.add_argument('-i','--inputm',help="input file name", dest = 'inFileMonth',nargs=12, action = 'store',)
parser.add_argument('-v','--variable',help="variable name",dest='var')
parser.add_argument('-y','--year',help="tested year",dest='year')
args = parser.parse_args()


y = int(args.year)


if y%4 == 0 and y%100 != 0 and y%400 != 0:
	leapyear = 'yes'
elif y%400 == 0:
	leapyear = 'yes'
else:
	leapyear = 'no'




#initialise arrays storing the results: original means, meanpres means
mm_original = np.zeros([12,48,96])
mm_meanpres = np.zeros([12,48,96])


#reads monthly output, safes selected variable
getvariable_month = {}
for i in range(12):
	f = nc.netcdf_file(args.inFileMonth[i],mmap=0)
	getvariable_month[i] = f.variables[args.var][:,:,:]

	if i == 0:
		lats = f.variables['lat'][:]
		lons = f.variables['lon'][:]

	f.close()

time = np.arange(12)


#safe original means in mm_original
for i in range(48):
	for j in range(96):	
		for k in range(12):
			mm_original[k,i,j] = getvariable_month[k][0,i,j]
			

#intialisise calendar, taking in account the leapyear
if leapyear == 'yes':
	caltod = np.array([31,29,31,30,31,30,31,31,30,31,30,31])
	#add a day in cal126 february
	cal126 = np.array([33,33,31,30,29,28,28,29,29,31,32,33])
	bias,timtod,tim126 = calcal(caltod,cal126)

elif leapyear == 'no':
	caltod = np.array([31,28,31,30,31,30,31,31,30,31,30,31])
	cal126 = np.array([33,32,31,30,29,28,28,29,29,31,32,33])
	bias,timtod,tim126 = calcal(caltod,cal126)

print('leapyear?')
print(leapyear)



#generate conversion matrix using mean preserving method
meanpresmatrix = meanpres_transform(caltod,cal126,timtod,tim126,leapyear)

for i in range(48):
	for j in range(96):
		#calculate new monthly means using the mean preserving method
		mm_meanpres[:,i,j] = np.dot(meanpresmatrix,mm_original[:,i,j])

			


#create netcdf files storing the new averaged data and old averaged including their norm variation
#new averaged using mean preserving method
ms = xr.Dataset({args.var: (['time','x','y'],mm_meanpres)},coords={'lat':(['x'],lats),'lon':(['y'],lons),'time':np.arange(0,12,1)})
ms.to_netcdf("results/meanpres_"+args.year+"_"+args.var+".nc",format='NETCDF3_CLASSIC')







