
from __future__ import division
from scipy.io import netcdf as nc
import scipy.sparse
import os
import sys
import numpy as np
import argparse
import xarray as xr
from matplotlib import pyplot as plt
from numpy import linalg as la


"""
Python Script for computing monthly means on an angular calendar for 126 ka.
The script is executed using the shell script newmean_bash_daily.

A parser is created, getting the desired year and variable and 12 daily files per year.

The subfunctions calcal and daily are described in detail below.

Output netcdf file:

daily_$year_$var.nc (Contains the exact new means)

"""



def calcal(caltod,calpast):

	"""
	A few calendar calculations: In 'bias' you find the calendar Offset.
	For example: bias[0] = -7 means that past's January started 7 days
	earlier w.r.t to the present calendar.
	In timtod/timtoday you find the absolute starting days, again w.r.t to the present calendar.
	Timpast can look like [-7,23,52,...], while timtod looks like [0,31,59,...]. 
	"""

	
	bias = np.zeros(12)
	timtod = np.zeros(13)
	timpast = np.zeros(13)

	bias[0] = caltod[0]+caltod[1]+caltod[2]-(calpast[0]+calpast[1]+calpast[2])

	for i in range(1,12):

		bias[i] = (bias[i-1]+calpast[i-1])-caltod[i-1]

	for i in range(1,13):

		timtod[i] = sum(caltod[:i])
		timpast[i] = sum(calpast[:i])

	timpast = timpast+bias[0]
	
	return bias,timtod,timpast




def fun_daily(caltod,calpast,timtod,timpast,getvariable_daily,leapyear):

	"""
	If daily data is available, this function computes new monthly means on the new calendar directly. 

	"""

	mm_daily = np.zeros([12,48,96])

	for i in range(48):

		for j in range(96):

			for k in range(12):

				if leapyear == 'yes':

					var = 366
				else:
					var = 365

				i0 = int(timpast[k]%var)
				i1 = int(timpast[k+1]%var)
			
				if i0 > i1:

					mm_daily[k,i,j] = (sum(getvariable_daily[i0:,i,j])+sum(getvariable_daily[:i1,i,j]))/calpast[k]

				else:
					mm_daily[k,i,j] = sum(getvariable_daily[i0:i1,i,j])/calpast[k]

	return mm_daily



desc = """\
Parser for reading daily data
"""

parser = argparse.ArgumentParser(description=desc, formatter_class = argparse.RawDescriptionHelpFormatter)

parser.add_argument('-i','--input',help= "input file name", dest = 'inFileDaily',nargs=12, action = 'store',)
parser.add_argument('-v','--variable',help="variable name",dest='var')
parser.add_argument('-y','--year',help="tested year",dest='year')
args = parser.parse_args()


#initialise arrays storing the results: daily Outpuz
mm_daily = np.zeros([12,48,96])

#reads daily output, safes selected variable
f = nc.netcdf_file(args.inFileDaily[0],mmap=0)
getvariable_daily = f.variables[args.var][:,:,:]

lats = f.variables['lat'][:]
lons = f.variables['lon'][:]
time = np.arange(12)

f.close()

for i in range(1,12):

	f = nc.netcdf_file(args.inFileDaily[i],mmap=0)

	#concatenate in one array per year
	getvariable_daily = np.concatenate((getvariable_daily,f.variables[args.var][:,:,:]))

	#test if leapyear by observing february length
	if i == 1:

		if f.variables[args.var][:,:,:].shape[0] == 29:

			leapyear = 'yes'
		else:
			leapyear = 'no'

	f.close()
		

#intialisise calendar, taking in account the leapyear
if leapyear == 'yes':

	caltod = np.array([31,29,31,30,31,30,31,31,30,31,30,31])
	#add a day in cal126 february
	cal126 = np.array([33,33,31,30,29,28,28,29,29,31,32,33])
	bias,timtod,tim126 = calcal(caltod,cal126)

elif leapyear == 'no':

	caltod = np.array([31,28,31,30,31,30,31,31,30,31,30,31])
	cal126 = np.array([33,32,31,30,29,28,28,29,29,31,32,33])
	bias,timtod,tim126 = calcal(caltod,cal126)


#calculate exact new monthly means
mm_daily[:,:,:] = fun_daily(caltod,cal126,timtod,tim126,getvariable_daily,leapyear)
			

#create netcdf files storing the new averaged data and old averaged including their norm variation
#new averaged using daily data
ds = xr.Dataset({args.var: (['time','x','y'], mm_daily)},coords={'time':time,'lat':(['x'],lats),'lon':(['y'],lons)})
ds.to_netcdf("results/daily_"+args.year+"_"+args.var+".nc",format='NETCDF3_CLASSIC')




